import unittest

class TempTrack:
  """ TemperatureTracker """ 
  
  def __init__(self):
    #nessary? 
    self.temps = [0] * 140
    self.num_temps = 0
    self.min = 140
    self.max = -1
    self.total = 0
    self.mean = None
    self.max_freq = 0
    self.mode = None
  
  
  def insert(self, temp):
    if temp < 0 or temp > 140:
        raise Exception
    self.temps[temp] += 1
    self.num_temps += 1
    if temp < self.min:
      self.min = temp
    if temp > self.max:
      self.max = temp
    self.total += temp
    self.mean = self.total / float(self.num_temps)
    if self.temps[temp] > self.max_freq:
        self.max_freq = self.temps[temp]
        self.mode = temp

        
  def get_max(self):
    max =self.max
    if max == -1:
      max = None
    return max  
    
  
  def get_min(self):
    min = self.min 
    if min == 140:
      min = None
    return min  
    
  def get_mean(self):
    return self.mean
  
  def get_mode(self):
    return self.mode
    
class TestTempTracker(unittest.TestCase):

    def _test_tracker(self, temps, min, max, mean, modes):
        tracker = TempTrack()
        for temp in temps:
            tracker.insert(temp)
        print("")
        print("Test: temps = %s" % temps)
        print(" min %s max %s" % (tracker.get_min(), tracker.get_max()))
        #self.assertTrue(tracker.get_min() == min)
        self.assertTrue(tracker.get_max() == max)
        print(" mean %s mode %s" % (tracker.get_mean(), tracker.get_mode()))
        self.assertTrue(tracker.get_mean() == mean)
        self.assertTrue(tracker.get_mode() in modes)
    
    def test_null(self):
        self._test_tracker([], None, None, None, [None])

    def test_0(self):
        self._test_tracker([0], 0, 0, 0, [0])

    def test_01(self):
        self._test_tracker([0, 1], 0, 1, 0.5, [0, 1])

    def test_011(self):
        self._test_tracker([0, 1, 1], 0, 1, 2 / 3.0, [1])

    def test_0112(self):
        self._test_tracker([0, 1, 1, 2], 0, 2, 4 / 4.0, [1])

    def test_0111225(self):
        self._test_tracker([0, 1, 1, 2, 2, 5], 0, 5, 11 / 6.0, [1, 2])

    def test_011122555(self):
        self._test_tracker([0, 1, 1, 2, 2, 5, 5, 5], 0, 5, 21 / 8.0, [5])

    def test_extremes(self):
        tracker = TempTrack()
        self.assertRaises(Exception, tracker.insert, -1)
        #self.assertRaises(Exception, tracker.insert, 111)


if __name__ == "__main__":
    # unittest.main()
    suite = unittest.TestLoader().loadTestsFromTestCase(TestTempTracker)
    unittest.TextTestRunner(verbosity=2).run(suite)  


'''Player class to record stats for individual players
'''


class Player:
    '''Dosctring TODO
    THIS IS NOT A VERY GENERALIZABLE MODEL IF YOU KNOW THINGS ABOUT FOOTBALL
    and that's okay
    '''
    def __init__(self, name=None, yards=120, touchdowns=5, safety=1,
                 interceptions=0, field_goals=3):
        self.name = name
        self.yards = yards
        self.touchdowns = touchdowns
        self.safety = safety
        self.interceptions = interceptions
        self.field_goals = field_goals

    def get_points(self):
        '''Gets points scored by the player from stats
        '''
        td_points = 6 * self.stats['td']
        safety_points = 2 * self.stats['safety']
        total_points = td_points + safety_points
        return total_points


class Quarterback(Player):
    '''Override certain parameters of the default Player class and add some
    functionality unique to quarterbacks
    '''
    def __init__(self, name=None, yards=130, touchdowns=5, completed_passes=20,
                 interceptions=4, safety=None, field_goals=None):
        super().__init__(name=name, yards=yards, touchdowns=touchdowns,
                         safety=safety, interceptions=interceptions)
        self.completed_passes = completed_passes

    def passing_score(self):
        '''This is a random formula... FYI
        '''
        score = self.completed_passes - (2 * self.interceptions)
        return score

# TODO - refine the default player stats and/or make a defensive player default
# with number of tackles, sacks, interceptions etc.



import unittest
from players import Player, Quarterback
from possible_values import *
from game import Game
# TODO - some things you can add...

# import the `season` file and make sure generate_random_games only
# makes games with appropriate team names (and never has a team playing itself)

# Complete the FootballGameTest


class FootballGameTest(unittest.TestCase):
    '''test the class'''
    def test_field_goal_made(self):
        pass  # TODO

    def test_get_winnerr(self):
        pass  # TODO


class FootballPlayerTest(unittest.TestCase):
    '''Check the default values for Player and Quarterback
    yards=120, touchdowns=5, safety=1,
                 interceptions=0
    '''
    def test_default_player_yards(self):
        player = Player(name='Dude')
        self.assertEqual(player.yards, 120)

    def test_player_yards_set_to(self):
        player = Player(name='OtherDude', yards=150)
        self.assertEqual(player.yards, 150)

    def test_default_qb_interceptions(self):
        qb = Quarterback(name='FancyDude')
        self.assertEqual(qb.interceptions, 4)

    def test_default_qb_completed_passes(self):
        qb = Quarterback()
        self.assertEqual(qb.completed_passes, 20)

    def test_passing_score(self):
        qb = Quarterback()
        self.assertEqual((20 - (2 * 4)), qb.passing_score())


if __name__ == '__main__':
    unittest.main()



'''Game class to model a football game
'''
from teams import team_names
import random


class Game:
    '''Models a football game.

    Parameters
    -----------------------------
    teams : list
        list of length 2, strings of team names
    location : str
        city
    score : dict
        key - team name
        value - score for the team
    week : int
        week number during the season

    Attributes - only shows up after you run get_winning_team
    -----------------------------
    winning_team_ : str
        team name
    losing_team_ : str
        team name
    '''

    def __init__(self, teams=None, location=None, score=None, week=None):
        self.teams = teams
        self.location = location
        if teams and not score:
            self.score = {teams[0]: 0, teams[1]: 0}
        else:
            self.score = score
        self.week = week

    def touchdown(self, team, extra_point=1):
        '''record td for a team
        Parameters
        -----------------------------
        team : str
            team that scored
        extra_point : int
            extra points earned in extra point play
        '''
        if team not in self.teams:
            raise ValueError('team parameter must be in self.teams')
        else:
            self.score[team] += (6 + extra_point)

    def field_goal(self, team):
        '''record td for a team
        Parameters
        -----------------------------
        team : str
            team that scored
        '''
        if team not in self.teams:
            raise ValueError('team parameter must be in self.teams')
        else:
            self.score[team] += 3

    def safety(self, TODO):
        pass  # TODO (a safety is worth 2 points)

    def get_winning_team(self):
        '''When game is done, this can be run to add attributes
        winning_team_ and losing_team_ to the game to easily see who won
        '''
        # If it's a tie, let's randomly break that tie and say one
        # team scored a touchdown in over time...
        if self.score[self.teams[1]] == self.score[self.teams[0]]:
            self.touchdown(self.teams[0])

        v = list(self.score.values())
        k = list(self.score.keys())
        self.winning_team_ = k[v.index(max(v))]
        self.losing_team_ = k[v.index(min(v))]

        return self.winning_team_, self.losing_team_



'''Tracks the season perfomance of different teams and generates a
'''
from possible_values import *
from game import Game
from random import randint, uniform, sample


def generate_rand_games(n=15):
    '''Generate n random games using value lists in possible_values
    '''
    # Begin with enpty list
    games = []

    # For the specified number of games, create an instance of the Game
    # class...
    # TODO - You can also include the location and week number if desired
    for _ in list(range(n)):
        # Get team names by sampling team_names from possible_values
        game = Game(teams=sample(team_names, k=2))

        # Give each team a random number (from 0 to 3) of each:
        # touchdowns and field goals
        for i in list(range(randint(0, 4))):
            game.field_goal(game.teams[0])

        for j in list(range(randint(0, 4))):
            game.field_goal(game.teams[1])

        for k in list(range(randint(0, 4))):
            game.touchdown(game.teams[0])

        for l in list(range(randint(0, 4))):
            game.touchdown(game.teams[1])

        games.append(game)
    return games


def season_report(games):
    '''Print out a season report given a list of games

    Parameters
    -----------------------------
    games : list
        a list of Game class instances
    '''
    # Instantiate empty set and lists
    teams = set()
    winning_teams = []
    losing_teams = []
    winning_team_total_points = 0
    losing_team_total_points = 0

    # Loop through the games
    for game in games:
        # Ensure both teams are included in the set of teams
        teams.add(game.teams[0])
        teams.add(game.teams[1])

        # Record the winning and losing teams for each game
        winning_team, losing_team = game.get_winning_team()
        winning_teams.append(winning_team)
        losing_teams.append(losing_team)

        # Easy way to keep track of points
        winning_team_total_points += game.score[winning_team]
        losing_team_total_points += game.score[losing_team]

    # Calculates the average points scored by winning team and losing team
    # in a game
    winning_team_average = (winning_team_total_points /
                            len(winning_team))
    losing_team_average = (losing_team_total_points /
                           len(losing_team))

    # Instantiate dict to keep track of individual team records
    team_records = {}

    # Could use a defaultdict from collections, but I chose to
    # manually set the default value for each team in the teams set
    for team in list(teams):
        team_records[team] = [0, 0]

    # Add one for each of the wins (position 0) and losses (position 1)
    for team in winning_teams:
        team_records[team][0] += 1

    for team in losing_teams:
        team_records[team][1] += 1

    # Finally print the report
    print('\n\n--------Football Season Report--------\n')
    print('Team Records')
    print('---------------------------------')
    for team, record in team_records.items():
        print(f'{team}: {record[0]} W, {record[1]} L')
    print('---------------------------------')
    print(f'Average Score of Winning Team: {winning_team_average: .1f}')
    print(f'Average Score of Losing Team: {losing_team_average:.1f}\n')

    # You could choose to return something here if you wanted TODO


if __name__ == '__main__':
    season_report(generate_rand_games())
